#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest
import json

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, random_string, function, random_int
from testlib import EapiConfigUnitTest

import pyeapi.api.interfaces

INTERFACES = ['Ethernet1', 'Ethernet1/1', 'Vlan1234', 'Management1',
              'Port-Channel1', 'Vxlan1']


class TestFunctions(unittest.TestCase):

    def test_isvalidinterface_returns_true(self):
        func = pyeapi.api.interfaces.isvalidinterface
        for intf in INTERFACES:
            self.assertTrue(func(intf))

    def test_isvalidinterface_returns_false(self):
        func = pyeapi.api.interfaces.isvalidinterface
        for intf in ['Et1', 'Ma1', 'Po1', 'Vl1', random_string()]:
            self.assertFalse(func(intf))

    def test_instance(self):
        result = pyeapi.api.interfaces.instance(None)
        self.assertIsInstance(result, pyeapi.api.interfaces.Interfaces)


class TestApiInterfaces(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiInterfaces, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.interfaces.Interfaces(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get_interface_generic(self):
        for intf in ['Management1', 'Loopback0']:
            result = self.instance.get(intf)
            self.assertEqual(result['type'], 'generic')

    def test_get_interface_ethernet(self):
        result = self.instance.get('Ethernet1')
        self.assertEqual(result['type'], 'ethernet')

    def test_get_invalid_interface(self):
        result = self.instance.get('Foo1')
        self.assertEqual(result, None)

    def test_proxy_method_success(self):
        result = self.instance.set_sflow('Ethernet1', True)
        self.assertTrue(result)

    def test_proxy_method_raises_attribute_error(self):
        with self.assertRaises(AttributeError):
            self.instance.set_sflow('Management1', True)


class TestApiBaseInterface(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiBaseInterface, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.interfaces.BaseInterface(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get('Loopback0')
        values = dict(name='Loopback0', type='generic',
                      shutdown=False, description=None)
        self.assertEqual(result, values)

    def test_set_description_with_value(self):
        for intf in INTERFACES:
            value = random_string()
            cmds = ['interface %s' % intf, 'description %s' % value]
            func = function('set_description', intf, value)
            self.eapi_positive_config_test(func, cmds)

    def test_set_description_with_no_value(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'no description']
            func = function('set_description', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_description_with_default(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'default description']
            func = function('set_description', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_shutdown(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'shutdown']
            func = function('set_shutdown', intf, default=False, disable=False)
            self.eapi_positive_config_test(func, cmds)

    def test_set_shutdown_with_disable(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'no shutdown']
            func = function('set_shutdown', intf)
            self.eapi_positive_config_test(func, cmds)

    def test_set_shutdown_with_default(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'default shutdown']
            func = function('set_shutdown', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_encapsulation_non_subintf(self):
        cmds = ['interface Ethernet1', 'encapsulation dot1q vlan 4']
        func = function('set_encapsulation', 'Ethernet1', 4)
        self.eapi_exception_config_test(func, NotImplementedError,
                                        cmds)

    def test_set_encapsulation_non_supported_intf(self):
        cmds = ['interface Vlan1234', 'encapsulation dot1q vlan 4']
        func = function('set_encapsulation', 'Vlan1234', 4)
        self.eapi_exception_config_test(func, NotImplementedError,
                                        cmds)

    def test_set_encapsulation_ethernet_subintf(self):
        cmds = ['interface Ethernet1.1', 'encapsulation dot1q vlan 4']
        func = function('set_encapsulation', 'Ethernet1.1', 4)
        self.eapi_positive_config_test(func, cmds)

    def test_set_encapsulation_portchannel_subintf_disable(self):
        cmds = ['interface Port-Channel1.1', 'no encapsulation dot1q vlan']
        func = function('set_encapsulation', 'Port-Channel1.1', 4,
                        disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_encapsulation_ethernet_subintf_default(self):
        cmds = ['interface Ethernet1.1', 'default encapsulation dot1q vlan']
        func = function('set_encapsulation', 'Ethernet1.1', 4,
                        default=True)
        self.eapi_positive_config_test(func, cmds)


class TestApiEthernetInterface(EapiConfigUnitTest):

    INTERFACES = ['Ethernet1', 'Ethernet1/1']

    def __init__(self, *args, **kwargs):
        super(TestApiEthernetInterface, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.interfaces.EthernetInterface(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get('Ethernet1')
        values = dict(name='Ethernet1', type='ethernet',
                      description=None, shutdown=False,
                      sflow=True, flowcontrol_send='off',
                      flowcontrol_receive='off')
        self.assertEqual(values, result)

    def test_instance_functions(self):
        for intf in self.INTERFACES:
            for name in ['create', 'delete', 'default']:
                if name == 'create':
                    # Test create for subinterfaces
                    subintf = intf + '.1'
                    cmds = ['interface %s' % subintf]
                    func = function(name, subintf)
                    self.eapi_positive_config_test(func, cmds)
                elif name == 'delete':
                    # Test delete for subinterfaces
                    subintf = intf + '.1'
                    cmds = ['no interface %s' % subintf]
                    func = function(name, subintf)
                    self.eapi_positive_config_test(func, cmds)
                elif name == 'default':
                    cmds = 'default interface %s' % intf
                    func = function(name, intf)
                    self.eapi_positive_config_test(func, cmds)

    def test_instance_functions_exceptions(self):
        intf = 'Ethernet1'
        for name in ['create', 'delete']:
            if name == 'create':
                cmds = 'interface %s' % intf
                func = function(name, intf)
                self.eapi_exception_config_test(func, NotImplementedError,
                                                cmds)
            elif name == 'delete':
                cmds = 'no interface %s' % intf
                func = function(name, intf)
                self.eapi_exception_config_test(func, NotImplementedError,
                                                cmds)

    def test_set_flowcontrol_with_value(self):
        for intf in self.INTERFACES:
            for direction in ['send', 'receive']:
                for value in ['on', 'off']:
                    cmds = ['interface %s' % intf,
                            'flowcontrol %s %s' % (direction, value)]
                    func = function('set_flowcontrol', intf, direction, value)
                    self.eapi_positive_config_test(func, cmds)

    def test_set_flowcontrol_with_invalid_direction_raises_value_error(self):
        for intf in self.INTERFACES:
            func = function('set_flowcontrol', intf, 'invalid', None)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_flowcontrol_with_invalid_value_raises_value_error(self):
        for intf in self.INTERFACES:
            for direction in ['send', 'receive']:
                func = function('set_flowcontrol', intf, direction, 'invalid')
                self.eapi_exception_config_test(func, ValueError)

    def test_set_flowcontrol_with_no_value(self):
        for intf in self.INTERFACES:
            for direction in ['send', 'receive']:
                cmds = ['interface %s' % intf, 'no flowcontrol %s' % direction]
                func = function('set_flowcontrol', intf, direction)
                self.eapi_positive_config_test(func, cmds)

    def test_set_flowcontrol_with_disable(self):
        for intf in self.INTERFACES:
            for direction in ['send', 'receive']:
                cmds = ['interface %s' % intf, 'no flowcontrol %s' % direction]
                func = function('set_flowcontrol', intf, direction,
                                disable=True)
                self.eapi_positive_config_test(func, cmds)

    def test_set_flowcontrol_with_default(self):
        for intf in self.INTERFACES:
            for direction in ['send', 'receive']:
                cmds = ['interface %s' % intf,
                        'default flowcontrol %s' % direction]
                func = function('set_flowcontrol', intf, direction,
                                default=True)
                self.eapi_positive_config_test(func, cmds)

    def test_set_sflow_with_value(self):
        for intf in self.INTERFACES:
            for value in [True, False]:
                cmds = ['interface %s' % intf]
                if value:
                    cmds.append('sflow enable')
                else:
                    cmds.append('no sflow enable')
                func = function('set_sflow', intf, value)
                self.eapi_positive_config_test(func, cmds)

    def test_set_sflow_with_no_value(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'no sflow enable']
            func = function('set_sflow', intf)
            self.eapi_positive_config_test(func, cmds)

    def test_set_sflow_with_default(self):
        for intf in INTERFACES:
            cmds = ['interface %s' % intf, 'default sflow enable']
            func = function('set_sflow', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_sflow_invalid_value_raises_value_error(self):
        for intf in INTERFACES:
            func = function('set_sflow', intf, random_string())
            self.eapi_exception_config_test(func, ValueError)

    def test_set_vrf(self):
        for intf in INTERFACES:
            vrf = 'testvrf'
            cmds = ['interface %s' % intf, 'vrf forwarding %s' % vrf]
            func = function('set_vrf', intf, vrf)
            self.eapi_positive_config_test(func, cmds)


class TestApiPortchannelInterface(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiPortchannelInterface, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.interfaces.PortchannelInterface(None)
        self.config = open(get_fixture('running_config.portchannel')).read()

    def setUp(self):
        super(TestApiPortchannelInterface, self).setUp()
        response = open(get_fixture('show_portchannel.json'))
        self.node.enable.return_value = json.load(response)
        response.close()

    def test_get(self):
        result = self.instance.get('Port-Channel1')
        values = dict(name='Port-Channel1', type='portchannel',
                      description=None, shutdown=False,
                      lacp_mode='on', minimum_links=0,
                      lacp_fallback='disabled', lacp_timeout=90,
                      members=['Ethernet5', 'Ethernet6'])

        self.assertEqual(values, result)

    def test_set_minimum_links_with_value(self):
        minlinks = random_int(1, 16)
        cmds = ['interface Port-Channel1',
                'port-channel min-links %s' % minlinks]
        func = function('set_minimum_links', 'Port-Channel1', minlinks)
        self.eapi_positive_config_test(func, cmds)

    def test_set_minimum_links_with_no_value(self):
        cmds = ['interface Port-Channel1', 'no port-channel min-links']
        func = function('set_minimum_links', 'Port-Channel1')
        self.eapi_positive_config_test(func, cmds)

    def test_set_minimum_links_with_default(self):
        cmds = ['interface Port-Channel1', 'default port-channel min-links']
        func = function('set_minimum_links', 'Port-Channel1', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_minimum_links_with_disable(self):
        cmds = ['interface Port-Channel1', 'no port-channel min-links']
        func = function('set_minimum_links', 'Port-Channel1', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_lacp_timeout_with_value(self):
        timeout = random_int(1, 16)
        cmds = ['interface Port-Channel1', 'port-channel lacp fallback timeout %s' % timeout]
        func = function('set_lacp_timeout', 'Port-Channel1', timeout)
        self.eapi_positive_config_test(func, cmds)

    def test_set_lacp_fallback_with_individual(self):
        cmds = ['interface Port-Channel1', 'port-channel lacp fallback individual']
        func = function('set_lacp_fallback', 'Port-Channel1', 'individual')
        self.eapi_positive_config_test(func, cmds)

    def test_set_lacp_fallback_with_static(self):
        cmds = ['interface Port-Channel1', 'port-channel lacp fallback static']
        func = function('set_lacp_fallback', 'Port-Channel1', 'static')
        self.eapi_positive_config_test(func, cmds)

    def test_set_lacp_fallback_with_disabled(self):
        cmds = ['interface Port-Channel1', 'no port-channel lacp fallback']
        func = function('set_lacp_fallback', 'Port-Channel1', 'disabled')
        self.eapi_positive_config_test(func, cmds)

    def test_set_lacp_fallback_invalid_mode(self):
        func = function('set_lacp_fallback', 'Port-Channel1', random_string())
        self.eapi_negative_config_test(func)

    def test_get_lacp_mode(self):
        result = self.instance.get_lacp_mode('Port-Channel1')
        self.assertEqual(result, 'on')

    def test_get_members(self):
        result = self.instance.get_members('Port-Channel1')
        self.assertEqual(result, ['Ethernet5', 'Ethernet6'])

    def test_set_members(self):
        cmds = ['interface Ethernet6', 'no channel-group 1',
                'interface Ethernet7', 'channel-group 1 mode on']
        func = function('set_members', 'Port-Channel1',
                        ['Ethernet5', 'Ethernet7'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_members_same_mode(self):
        cmds = ['interface Ethernet6', 'no channel-group 1',
                'interface Ethernet7', 'channel-group 1 mode on']
        func = function('set_members', 'Port-Channel1',
                        ['Ethernet5', 'Ethernet7'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_members_update_mode(self):
        cmds = ['interface Ethernet6', 'no channel-group 1',
                'interface Ethernet7', 'channel-group 1 mode active']
        func = function('set_members', 'Port-Channel1',
                        ['Ethernet5', 'Ethernet7'], mode='active')
        self.eapi_positive_config_test(func, cmds)

    def test_set_members_mode_none(self):
        cmds = ['interface Ethernet6', 'no channel-group 1',
                'interface Ethernet7', 'channel-group 1 mode on']
        func = function('set_members', 'Port-Channel1',
                        ['Ethernet5', 'Ethernet7'], mode=None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_members_no_changes(self):
        func = function('set_members', 'Port-Channel1',
                        ['Ethernet5', 'Ethernet6'])
        self.eapi_positive_config_test(func)

    def test_set_lacp_mode(self):
        cmds = ['interface Ethernet5', 'no channel-group 1',
                'interface Ethernet6', 'no channel-group 1',
                'interface Ethernet5', 'channel-group 1 mode active',
                'interface Ethernet6', 'channel-group 1 mode active']
        func = function('set_lacp_mode', 'Port-Channel1', 'active')
        self.eapi_positive_config_test(func, cmds)

    def test_set_lacp_mode_invalid_mode(self):
        func = function('set_lacp_mode', 'Port-Channel1', random_string())
        self.eapi_negative_config_test(func)


class TestApiVxlanInterface(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiVxlanInterface, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.interfaces.VxlanInterface(None)
        self.config = open(get_fixture('running_config.vxlan')).read()

    def test_get(self):
        keys = ['name', 'type', 'description', 'shutdown', 'source_interface',
                'multicast_group', 'udp_port', 'vlans', 'flood_list',
                'multicast_decap']
        result = self.instance.get('Vxlan1')
        self.assertEqual(sorted(keys), sorted(result.keys()))

    def test_set_source_interface_with_value(self):
        cmds = ['interface Vxlan1', 'vxlan source-interface Loopback0']
        func = function('set_source_interface', 'Vxlan1', 'Loopback0')
        self.eapi_positive_config_test(func, cmds)

    def test_set_source_interface_with_no_value(self):
        cmds = ['interface Vxlan1', 'no vxlan source-interface']
        func = function('set_source_interface', 'Vxlan1', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_source_interface_with_default(self):
        cmds = ['interface Vxlan1', 'default vxlan source-interface']
        func = function('set_source_interface', 'Vxlan1', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_multicast_group_with_value(self):
        cmds = ['interface Vxlan1', 'vxlan multicast-group 239.10.10.10']
        func = function('set_multicast_group', 'Vxlan1', '239.10.10.10')
        self.eapi_positive_config_test(func, cmds)

    def test_set_multicast_group_with_no_value(self):
        cmds = ['interface Vxlan1', 'no vxlan multicast-group']
        func = function('set_multicast_group', 'Vxlan1', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_multicast_group_with_default(self):
        cmds = ['interface Vxlan1', 'default vxlan multicast-group']
        func = function('set_multicast_group', 'Vxlan1', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_multicast_decap(self):
        cmds = ['interface Vxlan1', 'vxlan multicast-group decap']
        func = function('set_multicast_decap', 'Vxlan1')
        self.eapi_positive_config_test(func, cmds)

    def test_set_multicast_decap_with_no_value(self):
        cmds = ['interface Vxlan1', 'no vxlan multicast-group decap']
        func = function('set_multicast_decap', 'Vxlan1', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_multicast_decap_with_default(self):
        cmds = ['interface Vxlan1', 'default vxlan multicast-group decap']
        func = function('set_multicast_decap', 'Vxlan1', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_udp_port_with_value(self):
        cmds = ['interface Vxlan1', 'vxlan udp-port 1024']
        func = function('set_udp_port', 'Vxlan1', '1024')
        self.eapi_positive_config_test(func, cmds)

    def test_set_udp_port_with_no_value(self):
        cmds = ['interface Vxlan1', 'no vxlan udp-port']
        func = function('set_udp_port', 'Vxlan1', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_udp_port_with_default(self):
        cmds = ['interface Vxlan1', 'default vxlan udp-port']
        func = function('set_udp_port', 'Vxlan1', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_update_vlan(self):
        cmds = ['interface Vxlan1', 'vxlan vlan add 10 vni 10']
        func = function('update_vlan', 'Vxlan1', 10, 10)
        self.eapi_positive_config_test(func, cmds)

    def test_remove_vlan(self):
        cmds = ['interface Vxlan1', 'vxlan vlan remove 10 vni']
        func = function('remove_vlan', 'Vxlan1', 10)
        self.eapi_positive_config_test(func, cmds)

    def test_add_vtep(self):
        cmds = ['interface Vxlan1', 'vxlan flood vtep add 1.1.1.1']
        func = function('add_vtep', 'Vxlan1', '1.1.1.1')
        self.eapi_positive_config_test(func, cmds)

    def test_add_vtep_to_vlan(self):
        cmds = ['interface Vxlan1', 'vxlan vlan 10 flood vtep add 1.1.1.1']
        func = function('add_vtep', 'Vxlan1', '1.1.1.1', vlan='10')
        self.eapi_positive_config_test(func, cmds)

    def test_remove_vtep(self):
        cmds = ['interface Vxlan1', 'vxlan flood vtep remove 1.1.1.1']
        func = function('remove_vtep', 'Vxlan1', '1.1.1.1')
        self.eapi_positive_config_test(func, cmds)

    def test_remove_vtep_from_vlan(self):
        cmds = ['interface Vxlan1', 'vxlan vlan 10 flood vtep remove 1.1.1.1']
        func = function('remove_vtep', 'Vxlan1', '1.1.1.1', vlan='10')
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



import unittest
import collections

from mock import patch, Mock

import pyeapi.utils


class TestUtils(unittest.TestCase):

    @patch('pyeapi.utils.import_module')
    def test_load_module(self, mock_import_module):
        loaded_module = Mock(object='loaded_module')
        mock_import_module.return_value = loaded_module
        result = pyeapi.utils.load_module('test')
        self.assertEqual(result, loaded_module)

    @patch('pyeapi.utils.import_module')
    def test_load_module_raises_import_error(self, mock_import_module):
        mock_import_module.return_value = None
        with self.assertRaises(ImportError):
            pyeapi.utils.load_module('test')

    def test_make_iterable_from_string(self):
        result = pyeapi.utils.make_iterable('test')
        self.assertIsInstance(result, collections.Iterable)
        self.assertEqual(len(result), 1)

    def test_make_iterable_from_unicode(self):
        result = pyeapi.utils.make_iterable(u'test')
        self.assertIsInstance(result, collections.Iterable)
        self.assertEqual(len(result), 1)

    def test_make_iterable_from_iterable(self):
        result = pyeapi.utils.make_iterable(['test'])
        self.assertIsInstance(result, collections.Iterable)
        self.assertEqual(len(result), 1)

    def test_make_iterable_raises_type_error(self):
        with self.assertRaises(TypeError):
            pyeapi.utils.make_iterable(object())

    def test_import_module(self):
        result = pyeapi.utils.import_module('pyeapi.api.vlans')
        self.assertIsNotNone(result)

    def test_import_module_raises_import_error(self):
        with self.assertRaises(ImportError):
            pyeapi.utils.import_module('fake.module.test')

    def test_expand_singles(self):
        vlans = '1,2,3'
        result = pyeapi.utils.expand_range(vlans)
        result = ','.join(result)
        self.assertTrue(vlans == result)

    def test_expand_range(self):
        vlans = '1-15'
        expected = [str(x) for x in range(1, 16)]
        result = pyeapi.utils.expand_range(vlans)
        self.assertEqual(result, expected)

    def test_expand_mixed(self):
        vlans = '1,3,5-7,9'
        result = pyeapi.utils.expand_range(vlans)
        self.assertEqual(result, ['1', '3', '5', '6', '7', '9'])

    def test_collapse_singles(self):
        vlans = '1,3,5,7'
        result = pyeapi.utils.collapse_range(vlans)
        self.assertEqual(result, ['1', '3', '5', '7'])

    def test_collapse_range(self):
        vlans = '1,2,3,4,5'
        result = pyeapi.utils.collapse_range(vlans)
        self.assertEqual(result, ['1-5'])

    def test_collapse_mixed(self):
        vlans = '1,3,5,6,7,9'
        result = pyeapi.utils.collapse_range(vlans)
        self.assertEqual(result, ['1', '3', '5-7', '9'])

    @patch('pyeapi.utils._LOGGER')
    def test_debug(self, mock_logger):
        pyeapi.utils.islocalconnection = Mock(return_value=True)
        pyeapi.utils.debug('test')
        mock_logger.debug.assert_called_with('test_utils.test_debug: test')



#!/usr/bin/env python
from __future__ import print_function
import pyeapi

pyeapi.load_config('nodes.conf')
node = pyeapi.connect_to('veos01')

print('Show running-config for veos01')
print(('-'*30))
print((node.get_config('running-config')))
print()
print()

print('Show startup-config for veos01')
print(('-'*30))
print((node.get_config('startup-config')))
print()
print()

print('Show config diffs')
print(('-'*30))
print((node.get_config('running-config', 'diffs')))
print()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.mlag

class TestApiMlag(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiMlag, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.mlag.instance(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get()

        keys = ['config', 'interfaces']

        intfkeys = ['mlag_id']
        interfaces = result['interfaces']['Port-Channel10']

        cfgkeys = ['domain_id', 'local_interface', 'peer_address',
                   'peer_link', 'shutdown']

        self.assertEqual(sorted(keys), sorted(result.keys()))
        self.assertEqual(sorted(cfgkeys), sorted(result['config'].keys()))
        self.assertEqual(sorted(intfkeys), sorted(interfaces.keys()))

    def test_set_domain_id(self):
        for state in ['config', 'negate', 'default']:
            cmds = ['mlag configuration']
            if state == 'config':
                cmds.append('domain-id test.dom-id string')
                func = function('set_domain_id', 'test.dom-id string')
            elif state == 'negate':
                cmds.append('no domain-id')
                func = function('set_domain_id', value='test', disable=True)
            elif state == 'default':
                cmds.append('default domain-id')
                func = function('set_domain_id', value='test', default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_local_interface(self):
        for state in ['config', 'negate', 'default']:
            cmds = ['mlag configuration']
            if state == 'config':
                cmds.append('local-interface Vlan1234')
                func = function('set_local_interface', 'Vlan1234')
            elif state == 'negate':
                cmds.append('no local-interface')
                func = function('set_local_interface', disable=True)
            elif state == 'default':
                cmds.append('default local-interface')
                func = function('set_local_interface', value='Vlan1234',
                                default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_peer_address(self):
        for state in ['config', 'negate', 'default']:
            cmds = ['mlag configuration']
            if state == 'config':
                cmds.append('peer-address 1.2.3.4')
                func = function('set_peer_address', '1.2.3.4')
            elif state == 'negate':
                cmds.append('no peer-address')
                func = function('set_peer_address', disable=True)
            elif state == 'default':
                cmds.append('default peer-address')
                func = function('set_peer_address', value='1.2.3.4',
                                default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_peer_link(self):
        for state in ['config', 'negate', 'default']:
            cmds = ['mlag configuration']
            if state == 'config':
                cmds.append('peer-link Ethernet1')
                func = function('set_peer_link', 'Ethernet1')
            elif state == 'negate':
                cmds.append('no peer-link')
                func = function('set_peer_link', disable=True)
            elif state == 'default':
                cmds.append('default peer-link')
                func = function('set_peer_link', value='Ethernet1',
                                default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_shutdown(self):
        for state in ['config', 'negate', 'default']:
            cmds = ['mlag configuration']
            if state == 'config':
                cmds.append('shutdown')
                func = function('set_shutdown', default=False, disable=False)
            elif state == 'negate':
                cmds.append('no shutdown')
                func = function('set_shutdown', disable=True)
            elif state == 'default':
                cmds.append('default shutdown')
                func = function('set_shutdown', default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_mlag_id(self):
        for state in ['config', 'negate', 'default']:
            if state == 'config':
                cmds = ['interface Ethernet1', 'mlag 1']
                func = function('set_mlag_id', 'Ethernet1', '1')
            elif state == 'negate':
                cmds = ['interface Ethernet1', 'no mlag']
                func = function('set_mlag_id', 'Ethernet1', disable=True)
            elif state == 'default':
                cmds = ['interface Ethernet1', 'default mlag']
                func = function('set_mlag_id', 'Ethernet1', value='1',
                                default=True)
            self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.bgp


class TestApiBgp(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiBgp, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.bgp.instance(None)
        self.config = open(get_fixture('running_config.bgp')).read()

    def test_get(self):
        result = self.instance.get()
        keys = ['bgp_as', 'router_id', 'maximum_paths', 'maximum_ecmp_paths',
                'shutdown', 'neighbors', 'networks']
        self.assertEqual(sorted(keys), sorted(result.keys()))

    def test_create(self):
        for bgpas in ['65000', 65000]:
            func = function('create', bgpas)
            cmds = 'router bgp {}'.format(bgpas)
            self.eapi_positive_config_test(func, cmds)

    def test_create_invalid_as(self):
        for bgpas in ['66000', 66000]:
            with self.assertRaises(ValueError):
                self.instance.create(bgpas)

    def test_delete(self):
        func = function('delete')
        cmds = 'no router bgp 65000'
        self.eapi_positive_config_test(func, cmds)

    def test_default(self):
        func = function('default')
        cmds = 'default router bgp 65000'
        self.eapi_positive_config_test(func, cmds)

    def test_add_network(self):
        func = function('add_network', '172.16.10.1', '24', 'test')
        cmds = ['router bgp 65000', 'network 172.16.10.1/24 route-map test']
        self.eapi_positive_config_test(func, cmds)

        func = function('add_network', '', '24', 'test')
        self.eapi_exception_config_test(func, ValueError)

        func = function('add_network', '172.16.10.1', '', 'test')
        self.eapi_exception_config_test(func, ValueError)

    def test_remove_network(self):
        func = function('remove_network', '172.16.10.1', '24', 'test')
        cmds = ['router bgp 65000', 'no network 172.16.10.1/24 route-map test']
        self.eapi_positive_config_test(func, cmds)

        func = function('remove_network', '', '24', 'test')
        self.eapi_exception_config_test(func, ValueError)

        func = function('remove_network', '172.16.10.1', '', 'test')
        self.eapi_exception_config_test(func, ValueError)

    def test_set_router_id(self):
        for state in ['config', 'negate', 'default']:
            rid = '1.1.1.1'
            if state == 'config':
                cmds = ['router bgp 65000', 'router-id 1.1.1.1']
                func = function('set_router_id', rid)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no router-id']
                func = function('set_router_id', None, False, True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default router-id']
                func = function('set_router_id', rid, True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_router_id', value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no router-id']
        func = function('set_router_id', None)
        self.eapi_positive_config_test(func, cmds)

    def test_maximum_paths_just_max_path(self):
        for state in ['config', 'negate', 'default']:
            max_paths = 20
            if state == 'config':
                cmds = ['router bgp 65000', 'maximum-paths 20']
                func = function('set_maximum_paths', max_paths)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no maximum-paths']
                func = function('set_maximum_paths', disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default maximum-paths']
                func = function('set_maximum_paths', default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_maximum_paths', max_path=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no maximum-paths']
        func = function('set_maximum_paths', None)
        self.eapi_positive_config_test(func, cmds)

    def test_maximum_paths_max_path_and_ecmp(self):
        for state in ['config', 'negate', 'default']:
            max_paths = 20
            max_ecmp_path = 20
            if state == 'config':
                cmds = ['router bgp 65000', 'maximum-paths 20 ecmp 20']
                func = function('set_maximum_paths', max_paths, max_ecmp_path)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no maximum-paths']
                func = function('set_maximum_paths', disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default maximum-paths']
                func = function('set_maximum_paths', default=True)
            self.eapi_positive_config_test(func, cmds)

        func = function('set_maximum_paths', max_path=None, max_ecmp_path=20,
                        default=False, disable=False)
        self.eapi_exception_config_test(func, TypeError)

    def test_set_shutdown(self):
        for state in ['config', 'negate', 'default']:
            if state == 'config':
                cmds = ['router bgp 65000', 'shutdown']
                func = function('set_shutdown', default=False, disable=False)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no shutdown']
                func = function('set_shutdown', disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default shutdown']
                func = function('set_shutdown', default=True)
            self.eapi_positive_config_test(func, cmds)


class TestApiBgpNeighbor(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiBgpNeighbor, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.bgp.BgpNeighbors(None)
        self.config = open(get_fixture('running_config.bgp')).read()

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)
        self.assertEqual(len(result), 3)

    def test_get(self):
        result = self.instance.get('test')
        keys = ['name', 'send_community', 'shutdown', 'description',
                'remote_as', 'next_hop_self', 'route_map_in', 'route_map_out',
                'peer_group']
        self.assertEqual(sorted(keys), sorted(result.keys()))

    def test_delete(self):
        cmds = ['router bgp 65000', 'no neighbor test']
        func = function('delete', 'test')
        self.eapi_positive_config_test(func, cmds)

    def test_set_peer_group(self):
        for state in ['config', 'negate', 'default']:
            peer_group = 'test'
            name = '172.16.10.1'
            cmd = 'neighbor {} peer-group'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} {}'.format(cmd, peer_group)]
                func = function('set_peer_group', name, peer_group)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {}'.format(cmd)]
                func = function('set_peer_group', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default {}'.format(cmd)]
                func = function('set_peer_group', name, peer_group,
                                default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_peer_group', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor 172.16.10.1 peer-group']
        func = function('set_peer_group', '172.16.10.1', None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_remote_as(self):
        for state in ['config', 'negate', 'default']:
            remote_as = '65000'
            name = 'test'
            cmd = 'neighbor {} remote-as'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} {}'.format(cmd, remote_as)]
                func = function('set_remote_as', name, remote_as)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {}'.format(cmd)]
                func = function('set_remote_as', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default {}'.format(cmd)]
                func = function('set_remote_as', name, remote_as, default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_remote_as', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor test remote-as']
        func = function('set_remote_as', 'test', None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_shutdown(self):
        for state in ['config', 'negate', 'default', 'false']:
            name = 'test'
            cmd = 'neighbor {}'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} shutdown'.format(cmd)]
                func = function('set_shutdown', name, default=False,
                                disable=False)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {} shutdown'.format(cmd)]
                func = function('set_shutdown', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default {} shutdown'.format(cmd)]
                func = function('set_shutdown', name, default=True)
            elif state == 'false':
                cmds = ['router bgp 65000', 'no {} shutdown'.format(cmd)]
                func = function('set_shutdown', name, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_send_community(self):
        for state in ['config', 'negate', 'default']:
            name = 'test'
            cmd = 'neighbor {}'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} send-community'.format(cmd)]
                func = function('set_send_community', name, value=True)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {} send-community'.format(cmd)]
                func = function('set_send_community', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000',
                        'default {} send-community'.format(cmd)]
                func = function('set_send_community', name, value=False,
                                default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_send_community', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor test send-community']
        func = function('set_send_community', 'test', None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_next_hop_self(self):
        for state in ['config', 'negate', 'default']:
            name = 'test'
            cmd = 'neighbor {}'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} next-hop-self'.format(cmd)]
                func = function('set_next_hop_self', name, value=True)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {} next-hop-self'.format(cmd)]
                func = function('set_next_hop_self', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000',
                        'default {} next-hop-self'.format(cmd)]
                func = function('set_next_hop_self', name, value=False,
                                default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_next_hop_self', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor test next-hop-self']
        func = function('set_next_hop_self', 'test', None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_route_map_in(self):
        for state in ['config', 'negate', 'default']:
            route_map = 'TEST_RM'
            name = 'test'
            cmd = 'neighbor {} route-map'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} {} in'.format(cmd, route_map)]
                func = function('set_route_map_in', name, value=route_map)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {} in'.format(cmd)]
                func = function('set_route_map_in', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default {} in'.format(cmd)]
                func = function('set_route_map_in', name, value=route_map,
                                default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_route_map_in', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor test route-map in']
        func = function('set_route_map_in', 'test', None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_route_map_out(self):
        for state in ['config', 'negate', 'default']:
            route_map = 'TEST_RM'
            name = 'test'
            cmd = 'neighbor {} route-map'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} {} out'.format(cmd, route_map)]
                func = function('set_route_map_out', name, value=route_map)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {} out'.format(cmd)]
                func = function('set_route_map_out', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default {} out'.format(cmd)]
                func = function('set_route_map_out', name, value=route_map,
                                default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_route_map_out', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor test route-map out']
        func = function('set_route_map_out', 'test', None)
        self.eapi_positive_config_test(func, cmds)

    def test_set_description(self):
        for state in ['config', 'negate', 'default']:
            value = 'this is a test'
            name = 'test'
            cmd = 'neighbor {} description'.format(name)
            if state == 'config':
                cmds = ['router bgp 65000', '{} {}'.format(cmd, value)]
                func = function('set_description', name, value=value)
            elif state == 'negate':
                cmds = ['router bgp 65000', 'no {}'.format(cmd)]
                func = function('set_description', name, disable=True)
            elif state == 'default':
                cmds = ['router bgp 65000', 'default {}'.format(cmd)]
                func = function('set_description', name, value=value,
                                default=True)
            self.eapi_positive_config_test(func, cmds)

        # func = function('set_description', name, value=None, default=False,
        #                 disable=False)
        # self.eapi_exception_config_test(func, ValueError)
        # If command_builder fails because value is None, uncomment
        # above lines and remove below lines.
        cmds = ['router bgp 65000', 'no neighbor test description']
        func = function('set_description', 'test', None)
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest
import imp

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from mock import Mock, patch, call

from testlib import get_fixture, random_string, random_int

import pyeapi.client

DEFAULT_CONFIG = {'connection:localhost': dict(transport='socket')}


class TestNode(unittest.TestCase):

    def setUp(self):
        self.connection = Mock()
        self.node = pyeapi.client.Node(self.connection)

    def test_get_version_properties_match_version_number_no_match_model(self):
        self.node.enable = Mock()
        version = '4.17.1.1F-3512479.41711F (engineering build)'
        self.node.enable.return_value = [{'result': {'version': version,
                                                     'modelName': 'vEOS'}}]
        self.node._get_version_properties()
        self.assertEqual(self.node.version_number, '4.17.1.1')
        self.assertEqual(self.node.model, 'vEOS')

    def test_get_version_properties_no_match_version_number_match_model(self):
        self.node.enable = Mock()
        version = 'special-4.17.1.1F-3512479.41711F (engineering build)'
        model = 'DCS-7260QX-64-F'
        self.node.enable.return_value = [{'result': {'version': version,
                                                     'modelName': model}}]
        self.node._get_version_properties()
        self.assertEqual(self.node.version_number, version)
        self.assertEqual(self.node.model, '7260')

    def test_version_properties_populate(self):
        self.node.enable = Mock()
        version = '4.17.1.1F-3512479.41711F (engineering build)'
        self.node.enable.return_value = [{'result': {'version': version,
                                                     'modelName': 'vEOS'}}]
        self.node.version_number
        self.assertEqual(self.node.version_number, '4.17.1.1')
        self.assertEqual(self.node.version, version)
        self.assertEqual(self.node.model, 'vEOS')

    def test_enable_with_single_command(self):
        command = random_string()
        response = ['enable', command]

        self.connection.execute.return_value = {'result': list(response)}
        result = self.node.enable(command)

        self.connection.execute.assert_called_once_with(response, 'json')
        self.assertEqual(command, result[0]['result'])

    def test_enable_with_single_unicode_command(self):
        command = random_string()
        command = u'%s' % command
        response = ['enable', command]

        self.connection.execute.return_value = {'result': list(response)}
        result = self.node.enable(command)

        self.connection.execute.assert_called_once_with(response, 'json')
        self.assertEqual(command, result[0]['result'])

    def test_enable_with_single_extended_command(self):
        command = {'cmd': 'show cvx', 'revision': 2}
        response = ['enable', command]

        self.connection.execute.return_value = {'result': list(response)}
        result = self.node.enable(command)

        self.connection.execute.assert_called_once_with(response, 'json')
        self.assertEqual(command, result[0]['result'])

    def test_no_enable_with_single_command(self):
        command = random_string()
        response = [command]

        self.connection.execute.return_value = {'result': list(response)}
        result = self.node.enable(command, send_enable=False)

        self.connection.execute.assert_called_once_with(response, 'json')
        self.assertEqual(command, result[0]['result'])

    def test_enable_with_multiple_commands(self):
        commands = list()
        for i in range(0, random_int(2, 5)):
            commands.append(random_string())

        def execute_response(cmds, *args):
            return {'result': [x for x in cmds]}

        self.connection.execute.side_effect = execute_response

        responses = self.node.enable(commands)

        self.assertEqual(self.connection.execute.call_count, len(commands))

        expected_calls = [call(['enable', cmd], 'json') for cmd in commands]
        self.assertEqual(self.connection.execute.mock_calls, expected_calls)

        for index, response in enumerate(responses):
            self.assertEqual(commands[index], response['result'])

    def test_enable_with_multiple_unicode_commands(self):
        commands = list()
        for i in range(0, random_int(2, 5)):
            commands.append(u'%s' % random_string())

        def execute_response(cmds, *args):
            return {'result': [x for x in cmds]}

        self.connection.execute.side_effect = execute_response

        responses = self.node.enable(commands)

        self.assertEqual(self.connection.execute.call_count, len(commands))

        expected_calls = [call(['enable', cmd], 'json') for cmd in commands]
        self.assertEqual(self.connection.execute.mock_calls, expected_calls)

        for index, response in enumerate(responses):
            self.assertEqual(commands[index], response['result'])

    def test_config_with_single_command(self):
        command = random_string()
        self.node.run_commands = Mock(return_value=[{}, {}])
        result = self.node.config(command)
        self.assertEqual(result, [{}])

    def test_config_with_multiple_commands(self):
        commands = [random_string(), random_string()]
        self.node.run_commands = Mock(return_value=[{}, {}, {}])
        result = self.node.config(commands)
        self.assertEqual(result, [{}, {}])

    def test_config_with_single_multiline(self):
        command = ('banner login MULTILINE:This is a new banner\n'
                   'with different lines!!!')

        self.node.run_commands = Mock(return_value=[{}, {}])
        result = self.node.config(command)
        self.assertEqual(result, [{}])

    def test_config_with_multiple_multilines(self):
        commands = [random_string(),
                    ('banner login MULTILINE:This is a new banner\n'
                     'with different lines!!!'),
                    random_string()]

        self.node.run_commands = Mock(return_value=[{}, {}, {}, {}])
        result = self.node.config(commands)
        self.assertEqual(result, [{}, {}, {}])

    def test_get_config(self):
        config = [dict(output='test\nconfig')]
        self.node.run_commands = Mock(return_value=config)
        result = self.node.get_config()
        self.assertIsInstance(result, list)

    def test_get_config_as_string(self):
        config = [dict(output='test\nconfig')]
        self.node.run_commands = Mock(return_value=config)
        result = self.node.get_config(as_string=True)
        self.assertIsInstance(result, str)

    def test_get_config_raises_type_error(self):
        with self.assertRaises(TypeError):
            self.node.get_config('invalid-config')

    def test_api_autoloader(self):
        result = self.node.api('system')
        self.assertIsNotNone(result)

    def test_enable_authentication(self):
        self.assertIsNone(self.node._enablepwd)
        self.node.enable_authentication('test')
        self.assertEqual(self.node._enablepwd, 'test')

    def test_enable_with_config_statement(self):
        cmds = ['show version', 'configure', 'hostname foo']
        with self.assertRaises(TypeError):
            self.node.enable(cmds)


class TestClient(unittest.TestCase):

    def setUp(self):
        if 'EAPI_CONF' in os.environ:
            del os.environ['EAPI_CONF']
        imp.reload(pyeapi.client)

    def test_load_config_for_connection_with_filename(self):
        conf = get_fixture('eapi.conf')
        pyeapi.client.load_config(filename=conf)
        cfg = pyeapi.client.config.get_connection('test1')
        self.assertEqual(cfg['host'], '192.168.1.16')
        self.assertEqual(cfg['username'], 'eapi')
        self.assertEqual(cfg['password'], 'password')
        self.assertEqual(cfg['enablepwd'], 'enablepwd')

    def test_load_config_for_connection_with_env(self):
        os.environ['EAPI_CONF'] = get_fixture('eapi.conf')
        pyeapi.client.load_config(random_string())
        cfg = pyeapi.client.config.get_connection('test1')
        self.assertEqual(cfg['host'], '192.168.1.16')
        self.assertEqual(cfg['username'], 'eapi')
        self.assertEqual(cfg['password'], 'password')
        self.assertEqual(cfg['enablepwd'], 'enablepwd')

    def test_load_config(self):
        conf = get_fixture('eapi.conf')
        pyeapi.client.load_config(conf)
        self.assertEqual(len(pyeapi.client.config.sections()), 3)
        for name in ['localhost', 'test1', 'test2']:
            name = 'connection:%s' % name
            self.assertIn(name, pyeapi.client.config.sections())

    def test_load_config_empty_conf(self):
        conf = get_fixture('empty.conf')
        pyeapi.client.load_config(filename=conf)
        conns = pyeapi.client.config.connections
        self.assertEqual(conns, ['localhost'])

    def test_load_config_yaml(self):
        conf = get_fixture('eapi.conf.yaml')
        pyeapi.client.load_config(filename=conf)
        conns = pyeapi.client.config.connections
        self.assertEqual(conns, ['localhost'])

    def test_load_config_env_path(self):
        os.environ['EAPI_CONF'] = get_fixture('env_path.conf')
        pyeapi.client.config.autoload()
        self.assertIn('connection:env_path', pyeapi.client.config.sections())

    def test_config_always_has_default_connection(self):
        conf = '/invalid.conf'
        pyeapi.client.load_config(conf)
        self.assertEqual(len(pyeapi.client.config.sections()), 1)
        name = 'connection:localhost'
        self.assertIn(name, pyeapi.client.config.sections())

    def test_connections_property(self):
        conf = get_fixture('eapi.conf')
        pyeapi.client.load_config(conf)
        connections = ['test1', 'test2', 'localhost']
        result = pyeapi.client.config.connections
        self.assertEqual(sorted(connections), sorted(result))

    def test_missing_connection_raises_attribute_error(self):
        with self.assertRaises(AttributeError):
            pyeapi.client.connect_to('invalid')

    def test_config_for_replaces_host_w_name(self):
        conf = get_fixture('nohost.conf')
        pyeapi.client.load_config(conf)
        cfg = pyeapi.config_for('test')
        self.assertEqual(cfg['host'], 'test')

    def test_hosts_for_tag_returns_none(self):
        result = pyeapi.client.hosts_for_tag(random_string())
        self.assertIsNone(result)

    def test_hosts_for_tag_returns_names(self):
        conf = get_fixture('eapi.conf')
        pyeapi.client.load_config(conf)
        result = pyeapi.client.hosts_for_tag('tag1')
        self.assertEqual(sorted(['test1', 'test2']), sorted(result))

    @patch('pyeapi.client.make_connection')
    def test_connect_types(self, connection):
        transports = list(pyeapi.client.TRANSPORTS.keys())
        kwargs = dict(host='localhost', username='admin', password='',
                      port=None, key_file=None, cert_file=None,
                      ca_file=None, timeout=60)

        for transport in transports:
            pyeapi.client.connect(transport)
            connection.assert_called_with(transport, **kwargs)

    def test_make_connection_raises_typeerror(self):
        with self.assertRaises(TypeError):
            pyeapi.client.make_connection('invalid')

    def test_node_str_returns(self):
        node = pyeapi.client.Node(None)
        self.assertIsNotNone(str(node))

    def test_node_repr_returns(self):
        node = pyeapi.client.Node(None)
        self.assertIsNotNone(repr(node))

    def test_node_hasattr_connection(self):
        node = pyeapi.client.Node(None)
        self.assertTrue(hasattr(node, 'connection'))

    def test_node_returns_running_config(self):
        node = pyeapi.client.Node(None)
        get_config_mock = Mock(name='get_config')
        config_file = open(get_fixture('running_config.text'))
        config = config_file.read()
        config_file.close()
        get_config_mock.return_value = config
        node.get_config = get_config_mock
        self.assertIsInstance(node.running_config, str)

    def test_node_returns_startup_config(self):
        node = pyeapi.client.Node(None)
        get_config_mock = Mock(name='get_config')
        config_file = open(get_fixture('running_config.text'))
        config = config_file.read()
        config_file.close()
        get_config_mock.return_value = config
        node.get_config = get_config_mock
        self.assertIsInstance(node.startup_config, str)

    def test_node_returns_cached_startup_config(self):
        node = pyeapi.client.Node(None)
        config_file = open(get_fixture('running_config.text'))
        config = config_file.read()
        config_file.close()
        node._startup_config = config
        self.assertEqual(node.startup_config, config)

    def test_node_returns_version(self):
        node = pyeapi.client.Node(None)
        version = '4.17.1.1F-3512479.41711F (engineering build)'
        node.enable = Mock()
        node.enable.return_value = [{'result': {'version': version,
                                                'modelName': 'vEOS'}}]
        self.assertIsInstance(node.version, str)
        self.assertEqual(node.version, version)

    def test_node_returns_cached_version(self):
        node = pyeapi.client.Node(None)
        node._version = '4.16.7R'
        self.assertEqual(node.version, '4.16.7R')

    def test_node_returns_version_number(self):
        node = pyeapi.client.Node(None)
        version = '4.17.1.1F-3512479.41711F (engineering build)'
        node.enable = Mock()
        node.enable.return_value = [{'result': {'version': version,
                                                'modelName': 'vEOS'}}]
        self.assertIsInstance(node.version_number, str)
        self.assertIn(node.version_number, version)

    def test_node_returns_cached_version_number(self):
        node = pyeapi.client.Node(None)
        node._version_number = '4.16.7'
        self.assertEqual(node.version_number, '4.16.7')

    def test_node_returns_model(self):
        node = pyeapi.client.Node(None)
        version = '4.17.1.1F-3512479.41711F (engineering build)'
        model = 'DCS-7260QX-64-F'
        node.enable = Mock()
        node.enable.return_value = [{'result': {'version': version,
                                                'modelName': model}}]
        self.assertIsInstance(node.model, str)
        self.assertIn(node.model, model)

    def test_node_returns_cached_model(self):
        node = pyeapi.client.Node(None)
        node._model = '7777'
        self.assertEqual(node.model, '7777')

    def test_connect_default_type(self):
        transport = Mock()
        with patch.dict(pyeapi.client.TRANSPORTS, {'https': transport}):
            pyeapi.client.connect()
            kwargs = dict(host='localhost', username='admin', password='',
                          port=None, key_file=None, cert_file=None,
                          ca_file=None, timeout=60)
            transport.assert_called_once_with(**kwargs)

    def test_connect_return_node(self):
        transport = Mock()
        with patch.dict(pyeapi.client.TRANSPORTS, {'https': transport}):
            conf = get_fixture('eapi.conf')
            pyeapi.client.load_config(filename=conf)
            node = pyeapi.client.connect(host='192.168.1.16', username='eapi',
                                         password='password', port=None,
                                         timeout=60, return_node=True)
            kwargs = dict(host='192.168.1.16', username='eapi',
                          password='password', port=None, key_file=None,
                          cert_file=None, ca_file=None, timeout=60)
            transport.assert_called_once_with(**kwargs)
            self.assertIsNone(node._enablepwd)

    def test_connect_return_node_enablepwd(self):
        transport = Mock()
        with patch.dict(pyeapi.client.TRANSPORTS, {'https': transport}):
            conf = get_fixture('eapi.conf')
            pyeapi.client.load_config(filename=conf)
            node = pyeapi.client.connect(host='192.168.1.16', username='eapi',
                                         password='password', port=None,
                                         timeout=60, enablepwd='enablepwd',
                                         return_node=True)
            kwargs = dict(host='192.168.1.16', username='eapi',
                          password='password', port=None, key_file=None,
                          cert_file=None, ca_file=None, timeout=60)
            transport.assert_called_once_with(**kwargs)
            self.assertEqual(node._enablepwd, 'enablepwd')

    def test_connect_to_with_config(self):
        transport = Mock()
        with patch.dict(pyeapi.client.TRANSPORTS, {'https': transport}):
            conf = get_fixture('eapi.conf')
            pyeapi.client.load_config(filename=conf)
            node = pyeapi.client.connect_to('test1')
            kwargs = dict(host='192.168.1.16', username='eapi',
                          password='password', port=None, key_file=None,
                          cert_file=None, ca_file=None, timeout=60)
            transport.assert_called_once_with(**kwargs)
            self.assertEqual(node._enablepwd, 'enablepwd')


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, function
from testlib import EapiConfigUnitTest

import pyeapi.api.users

class TestApiUsers(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiUsers, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.users.instance(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_isprivilege_returns_false(self):
        result = pyeapi.api.users.isprivilege('test')
        self.assertFalse(result)

    def test_get(self):
        keys = ['nopassword', 'privilege', 'role', 'secret', 'format', 'sshkey']
        result = self.instance.get('test')
        self.assertEqual(sorted(keys), sorted(result.keys()))

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)

    def test_create_with_nopassword(self):
        cmds = 'username test nopassword'
        func = function('create', 'test', nopassword=True)
        self.eapi_positive_config_test(func, cmds)

    def test_create_with_secret_cleartext(self):
        cmds = 'username test secret 0 pass'
        func = function('create', 'test', secret='pass')
        self.eapi_positive_config_test(func, cmds)

    def test_create_with_secret_md5(self):
        cmds = 'username test secret 5 pass'
        func = function('create', 'test', secret='pass', encryption='md5')
        self.eapi_positive_config_test(func, cmds)

    def test_create_with_secret_sha512(self):
        cmds = 'username test secret sha512 pass'
        func = function('create', 'test', secret='pass', encryption='sha512')
        self.eapi_positive_config_test(func, cmds)

    def test_create_with_missing_kwargs(self):
        with self.assertRaises(TypeError):
            self.instance.create('test')

    def test_create_with_invalid_secret_arg(self):
        with self.assertRaises(TypeError):
            self.instance.create_with_secret('test', 'test', 'test')

    def test_delete(self):
        with self.assertRaises(TypeError):
            self.instance.delete('admin')

    def test_delete_admin_exception(self):
        cmds = 'no username test'
        func = function('delete', 'test')
        self.eapi_positive_config_test(func, cmds)

    def test_default(self):
        cmds = 'default username test'
        func = function('default', 'test')
        self.eapi_positive_config_test(func, cmds)

    def test_set_privilege(self):
        cmds = 'username test privilege 8'
        func = function('set_privilege', 'test', 8)
        self.eapi_positive_config_test(func, cmds)

    def test_set_privilege_negate(self):
        cmds = 'username test privilege 1'
        func = function('set_privilege', 'test')
        self.eapi_positive_config_test(func, cmds)

    def test_set_privilege_invalid_value(self):
        with self.assertRaises(TypeError):
            self.instance.set_privilege('test', 16)

    def test_set_role(self):
        cmds = 'username test role ops'
        func = function('set_role', 'test', value='ops')
        self.eapi_positive_config_test(func, cmds)

    def test_set_role_negate(self):
        cmds = 'no username test role'
        func = function('set_role', 'test', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_role_default(self):
        cmds = 'default username test role'
        func = function('set_role', 'test', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_sshkey(self):
        cmds = 'username test sshkey newkey'
        func = function('set_sshkey', 'test', value='newkey')
        self.eapi_positive_config_test(func, cmds)

    def test_set_sshkey_negate(self):
        cmds = 'no username test sshkey'
        func = function('set_sshkey', 'test', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_sshkey_default(self):
        cmds = 'default username test sshkey'
        func = function('set_sshkey', 'test', default=True)
        self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))


from testlib import get_fixture, random_vlan, function
from testlib import EapiConfigUnitTest

import pyeapi.api.switchports


class TestApiSwitchports(EapiConfigUnitTest):

    INTERFACES = ['Ethernet1', 'Ethernet1/1', 'Port-Channel1']

    def __init__(self, *args, **kwargs):
        super(TestApiSwitchports, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.switchports.instance(None)
        self.config = open(get_fixture('running_config.text')).read()

    def test_get(self):
        result = self.instance.get('Ethernet1')
        keys = ['name', 'mode', 'access_vlan', 'trunk_native_vlan',
                'trunk_allowed_vlans', 'trunk_groups']
        self.assertEqual(sorted(result.keys()), sorted(keys))

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)

    def test_instance_functions(self):
        for intf in self.INTERFACES:
            for name in ['create', 'delete', 'default']:
                if name == 'create':
                    cmds = ['interface %s' % intf, 'no ip address',
                            'switchport']

                elif name == 'delete':
                    cmds = ['interface %s' % intf, 'no switchport']

                elif name == 'default':
                    cmds = ['interface %s' % intf, 'no ip address',
                            'default switchport']

                func = function(name, intf)
                self.eapi_positive_config_test(func, cmds)

    def test_set_mode(self):
        for intf in self.INTERFACES:
            for mode in ['access', 'trunk']:
                cmds = ['interface %s' % intf, 'switchport mode %s' % mode]
                func = function('set_mode', intf, mode)
                self.eapi_positive_config_test(func, cmds)

    def test_set_mode_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no switchport mode']
            func = function('set_mode', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_mode_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'default switchport mode']
            func = function('set_mode', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_access_vlan(self):
        for intf in self.INTERFACES:
            vid = random_vlan()
            cmds = ['interface %s' % intf, 'switchport access vlan %s' % vid]
            func = function('set_access_vlan', intf, vid)
            self.eapi_positive_config_test(func, cmds)

    def test_set_access_vlan_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no switchport access vlan']
            func = function('set_access_vlan', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_access_vlan_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'default switchport access vlan']
            func = function('set_access_vlan', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_native_vlan(self):
        for intf in self.INTERFACES:
            vid = random_vlan()
            cmds = ['interface %s' % intf,
                    'switchport trunk native vlan %s' % vid]
            func = function('set_trunk_native_vlan', intf, vid)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_native_vlan_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no switchport trunk native vlan']
            func = function('set_trunk_native_vlan', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_native_vlan_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'default switchport trunk native vlan']
            func = function('set_trunk_native_vlan', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_allowed_vlans(self):
        for intf in self.INTERFACES:
            vid = '1,2,3-5,6,7'
            cmds = ['interface %s' % intf,
                    'switchport trunk allowed vlan %s' % vid]
            func = function('set_trunk_allowed_vlans', intf, vid)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_allowed_vlans_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'no switchport trunk allowed vlan']
            func = function('set_trunk_allowed_vlans', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_allowed_vlans_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'default switchport trunk allowed vlan']
            func = function('set_trunk_allowed_vlans', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'default switchport trunk group']
            func = function('set_trunk_groups', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_with_no(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'no switchport trunk group']
            func = function('set_trunk_groups', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_with_add(self):
        intf = 'Ethernet1'
        cmds = ['interface %s' % intf,
                'switchport trunk group bang']
        func = function('set_trunk_groups', intf, ['foo', 'bar', 'bang'])
        self.eapi_positive_config_test(func, cmds)

    def test_set_trunk_groups_with_remove(self):
        intf = 'Ethernet1'
        cmds = ['interface %s' % intf,
                'no switchport trunk group foo']
        func = function('set_trunk_groups', intf, ['bar'])
        self.eapi_positive_config_test(func, cmds)

    def test_add_trunk_group(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'switchport trunk group foo']
            func = function('add_trunk_group', intf, 'foo')
            self.eapi_positive_config_test(func, cmds)

    def test_remove_trunk_group(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf,
                    'no switchport trunk group foo']
            func = function('remove_trunk_group', intf, 'foo')
            self.eapi_positive_config_test(func, cmds)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
from __future__ import print_function
# import the client library
import pyeapi

# load the conf file and connect to the node
pyeapi.load_config('nodes.conf')
node = pyeapi.connect_to('veos01')

# get the vlan api and enable autorefresh
vlans = node.api('vlans')
vlans.autorefresh = True

if vlans.get(123):
    print('Vlan 123 already exists, deleting it')
    vlans.delete(1234)

print('\nCreating Vlan 123')
vlans.create(123)

print('Setting Vlan 123 name to "test_vlan"')
vlans.set_name(123, 'test_vlan')

print('\nDisplaying all vlans on the node')
print(('-'*30))
for vlan in list(vlans.values()):
    print(("   Vlan Id: {vlan_id}, Name: {name}".format(**vlan)))

if vlans.get(123):
    print('\nDeleting Vlan 123')
    vlans.delete(123)

print()





#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import random_int, random_string
from systestlib import DutSystemTest


class TestApiVlans(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no vlan 1', 'vlan 1'])
            response = dut.api('vlans').get('1')
            values = dict(vlan_id='1', name='default', state='active',
                          trunk_groups=[])
            self.assertEqual(values, response)

    def test_getall(self):
        for dut in self.duts:
            dut.config(['no vlan 1-4094', 'vlan 1', 'vlan 10'])
            response = dut.api('vlans').getall()
            self.assertIsInstance(response, dict, 'dut=%s' % dut)
            for vid in ['1', '10']:
                self.assertIn(vid, response, 'dut=%s' % dut)

    def test_create_and_return_true(self):
        for dut in self.duts:
            dut.config('no vlan 2-4094')
            vid = str(random_int(2, 4094))
            result = dut.api('vlans').create(vid)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan')
            self.assertIn(vid, config[0]['vlans'], 'dut=%s' % dut)

    def test_create_and_return_false(self):
        for dut in self.duts:
            result = dut.api('vlans').create('5000')
            self.assertFalse(result, 'dut=%s' % dut)

    def test_delete_and_return_true(self):
        for dut in self.duts:
            vid = str(random_int(2, 4094))
            dut.config('vlan %s' % vid)
            result = dut.api('vlans').delete(vid)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan')
            self.assertNotIn(vid, config[0]['vlans'])

    def test_delete_and_return_false(self):
        for dut in self.duts:
            result = dut.api('vlans').delete('5000')
            self.assertFalse(result, 'dut=%s' % dut)

    def test_default(self):
        for dut in self.duts:
            vid = str(random_int(2, 4095))
            name = random_string(maxchar=20)
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid,
                        'vlan %s' % vid, 'name %s' % name])
            result = dut.api('vlans').default(vid)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan')
            self.assertNotIn(vid, config[0]['vlans'], 'dut=%s' % dut)

    def test_set_name(self):
        for dut in self.duts:
            name = random_string(maxchar=20)
            vid = str(random_int(2, 4095))
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid])
            result = dut.api('vlans').set_name(vid, name)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan')
            self.assertEqual(name, config[0]['vlans'][vid]['name'],
                             'dut=%s' % dut)

    def test_set_state_active(self):
        for dut in self.duts:
            vid = str(random_int(2, 4095))
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid, 'state suspend'])
            result = dut.api('vlans').set_state(vid, 'active')
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan')
            self.assertEqual('active', config[0]['vlans'][vid]['status'],
                             'dut=%s' % dut)

    def test_set_state_suspend(self):
        for dut in self.duts:
            vid = str(random_int(2, 4095))
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid, 'state active'])
            result = dut.api('vlans').set_state(vid, 'suspend')
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan')
            self.assertEqual('suspended', config[0]['vlans'][vid]['status'],
                             'dut=%s' % dut)

    def test_set_trunk_groups_default(self):
        for dut in self.duts:
            vid = str(random_int(2, 4094))
            tg = random_string()
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid,
                        'trunk group %s' % tg])
            result = dut.api('vlans').set_trunk_groups(vid, default=True)
            self.assertTrue(result, 'dut=%s' % dut)
            cmd = 'show running-config section vlan %s' % vid
            config = dut.run_commands(cmd, 'text')
            self.assertNotIn('trunk group', config[0]['output'])

    def test_set_trunk_groups(self):
        for dut in self.duts:
            vid = str(random_int(2, 4094))
            tg1 = random_string(maxchar=10)
            tg2 = random_string(maxchar=10)
            tg3 = random_string(maxchar=10)
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid,
                        'trunk group %s' % tg1, 'trunk group %s' % tg2])
            result = dut.api('vlans').set_trunk_groups(vid, [tg1, tg3])
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan %s trunk group' % vid)
            config = sorted(config[0]['trunkGroups'][vid]['names'])
            self.assertEqual(sorted([tg1, tg3]), config)

    def test_set_trunk_groups_disable(self):
        for dut in self.duts:
            vid = str(random_int(2, 4094))
            tg1 = random_string(maxchar=10)
            tg2 = random_string(maxchar=10)
            tg3 = random_string(maxchar=10)
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid,
                        'trunk group %s' % tg1, 'trunk group %s' % tg2,
                        'trunk group %s' % tg3])
            result = dut.api('vlans').set_trunk_groups(vid, disable=True)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan %s trunk group' % vid)
            config = sorted(config[0]['trunkGroups'][vid]['names'])
            self.assertEqual([], config)

    def test_add_trunk_group(self):
        for dut in self.duts:
            tg = random_string(maxchar=32)
            vid = str(random_int(2, 4095))
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid, 'no trunk group'])
            result = dut.api('vlans').add_trunk_group(vid, tg)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan trunk group')
            self.assertIn(tg, config[0]['trunkGroups'][vid]['names'],
                          'dut=%s' % dut)

    def test_remove_trunk_group(self):
        for dut in self.duts:
            tg = random_string(maxchar=32)
            vid = str(random_int(2, 4095))
            dut.config(['no vlan %s' % vid, 'vlan %s' % vid, 'no trunk group',
                        'trunk group %s' % tg])
            result = dut.api('vlans').remove_trunk_group(vid, tg)
            self.assertTrue(result, 'dut=%s' % dut)
            config = dut.run_commands('show vlan trunk group')
            self.assertNotIn(tg, config[0]['trunkGroups'][vid]['names'],
                             'dut=%s' % dut)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import unittest
import random

from testlib import get_fixture

import pyeapi.client

class DutSystemTest(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(DutSystemTest, self).__init__(*args, **kwargs)
        self.longMessage = True

    def setUp(self):
        pyeapi.client.load_config(filename=get_fixture('dut.conf'))
        config = pyeapi.client.config

        self.duts = list()
        for name in config.sections():
            if name.startswith('connection:') and 'localhost' not in name:
                name = name.split(':')[1]
                self.duts.append(pyeapi.client.connect_to(name))

    def sort_dict_by_keys(self, d):
        keys = sorted(d.keys())
        return dict([(k, d[k]) for k in keys])


def random_interface(dut, exclude=None):
    exclude = [] if exclude is None else exclude
    interfaces = dut.api('interfaces')
    names = [name for name in list(interfaces.keys()) if name.startswith('Et')]

    exclude_interfaces = dut.settings.get('exclude_interfaces', [])
    if exclude_interfaces:
        exclude_interfaces = exclude_interfaces.split(',')
    exclude_interfaces.extend(exclude)

    if sorted(exclude_interfaces) == sorted(names):
        raise TypeError('unable to allocate interface from dut')

    choices = set(names).difference(exclude)
    return random.choice(list(choices))






#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import sys
import os
import unittest

sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import get_fixture, random_string, function
from testlib import EapiConfigUnitTest

import pyeapi.api.stp

def get_running_config():
    return get_fixture('running_config.text')

class TestApiStp(EapiConfigUnitTest):

    def __init__(self, *args, **kwargs):
        super(TestApiStp, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.stp.Stp(None)
        self.config = open(get_running_config()).read()

    def test_instance(self):
        result = pyeapi.api.stp.instance(None)
        self.assertIsInstance(result, pyeapi.api.stp.Stp)

    def test_interfaces(self):
        result = self.instance.interfaces
        self.assertIsInstance(result, pyeapi.api.stp.StpInterfaces)

    def test_instances(self):
        result = self.instance.instances
        self.assertIsInstance(result, pyeapi.api.stp.StpInstances)

    def test_set_mode_with_value(self):
        for value in ['mstp', 'none']:
            cmds = 'spanning-tree mode %s' % value
            func = function('set_mode', value)
            self.eapi_positive_config_test(func, cmds)

    def test_set_mode_with_default(self):
        cmds = 'default spanning-tree mode'
        func = function('set_mode', default=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_mode_with_disable(self):
        cmds = 'no spanning-tree mode'
        func = function('set_mode', disable=True)
        self.eapi_positive_config_test(func, cmds)

    def test_set_mode_invalid_value_raises_value_error(self):
        value = random_string()
        func = function('set_mode', value)
        self.eapi_exception_config_test(func, ValueError)

class TestApiStpInterfaces(EapiConfigUnitTest):

    INTERFACES = ['Ethernet1', 'Ethernet1/1', 'Port-Channel1']

    def __init__(self, *args, **kwargs):
        super(TestApiStpInterfaces, self).__init__(*args, **kwargs)
        self.instance = pyeapi.api.stp.StpInterfaces(None)
        self.config = open(get_running_config()).read()

    def test_getall(self):
        result = self.instance.getall()
        self.assertIsInstance(result, dict)

    def test_set_portfast_type_with_value(self):
        for intf in self.INTERFACES:
            for value in ['edge', 'network', 'normal']:
                cmds = ['interface %s' % intf]
                cmds.append('spanning-tree portfast %s' % value)
                if value == 'edge':
                    cmds.append('spanning-tree portfast auto')
                func = function('set_portfast_type', intf, value)
                self.eapi_positive_config_test(func, cmds)

    def test_set_portfast_type_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'spanning-tree portfast normal']
            func = function('set_portfast_type', intf)
            self.eapi_positive_config_test(func, cmds)

    def test_set_portfast_type_invalid_value_raises_value_error(self):
        for intf in self.INTERFACES:
            value = random_string()
            func = function('set_portfast_type', intf, value)
            self.eapi_exception_config_test(func, ValueError)

    def test_set_portfast_type_invalid_intf_raises_value_error(self):
        intf = random_string()
        func = function('set_portfast_type', intf)
        self.eapi_exception_config_test(func, ValueError)

    def test_set_bpduguard_with_value(self):
        for intf in self.INTERFACES:
            for value in ['enable', 'disable']:
                cfgvalue = value == 'enable'
                cmds = ['interface %s' % intf,
                        'spanning-tree bpduguard %s' % value]
                func = function('set_bpduguard', intf, cfgvalue)
                self.eapi_positive_config_test(func, cmds)

    def test_set_bpduguard_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'spanning-tree bpduguard disable']
            func = function('set_bpduguard', intf)
            self.eapi_positive_config_test(func, cmds)

    def test_set_bpduguard_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'default spanning-tree bpduguard']
            func = function('set_bpduguard', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_bpduguard_with_disable(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no spanning-tree bpduguard']
            func = function('set_bpduguard', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_bpduguard_invalid_intf_raises_value_error(self):
        intf = random_string()
        func = function('set_bpduguard', intf)
        self.eapi_exception_config_test(func, ValueError)

    def test_set_portfast_with_value(self):
        for intf in self.INTERFACES:
            for value in [True, False]:
                cmds = ['interface %s' % intf]
                if value:
                    cmds.append('spanning-tree portfast')
                else:
                    cmds.append('no spanning-tree portfast')
                func = function('set_portfast', intf, value)
                self.eapi_positive_config_test(func, cmds)

    def test_set_portfast_with_no_value(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'no spanning-tree portfast']
            func = function('set_portfast', intf, disable=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_portfast_with_default(self):
        for intf in self.INTERFACES:
            cmds = ['interface %s' % intf, 'default spanning-tree portfast']
            func = function('set_portfast', intf, default=True)
            self.eapi_positive_config_test(func, cmds)

    def test_set_portfast_invalid_intf_raises_value_error(self):
        intf = random_string()
        func = function('set_portfast', intf)
        self.eapi_exception_config_test(func, ValueError)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

from os import listdir, path, makedirs
from os.path import isfile, join, exists
import pprint as pp

HERE = path.abspath(path.dirname(__file__))

MODULES_PATH = '%s/../pyeapi/' % HERE
AUTOGEN = '.. This file has been autogenerated by generate_modules.py\n\n'


def get_module_names(p):
    '''Accepts a path to search for modules. The method will filter on files
    that end in .pyc or files that start with __.

    Arguments:
        p (string): The path to search
    Returns:
        list of file names
    '''
    mods = list()
    mods = [f.split('.')[0] for f in listdir(p)
            if isfile(join(p, f)) and not f.endswith('.pyc') and not f.startswith('__')]
    print len(mods)
    return mods

def process_modules(modules):
    '''Accepts dictionary of 'client' and 'api' modules and creates
    the corresponding files.
    '''
    for mod in modules['client']:
        directory = '%s/client_modules' % HERE
        if not exists(directory):
            makedirs(directory)
        write_module_file(mod, directory, 'pyeapi')

    for mod in modules['api']:
        directory = '%s/api_modules' % HERE
        if not exists(directory):
            makedirs(directory)
        write_module_file(mod, directory, 'pyeapi.api')

    create_index(modules)

def create_index(modules):
    '''This takes a dict of modules and created the RST index file.'''
    for key in modules.keys():
        file_path = join(HERE, '%s_modules/_list_of_modules.rst' % key)
        list_file = open(file_path, 'w')

        # Write the generic header
        list_file.write('%s\n' % AUTOGEN)
        list_file.write('%s\n' % key.title())
        list_file.write('=' * len(key))
        list_file.write('\n\n')
        list_file.write('.. toctree::\n')
        list_file.write('   :maxdepth: 2\n\n')

        for module in modules[key]:
            list_file.write('   %s\n' % module)


def write_module_file(name, path, package):
    '''Creates an RST file for the module name passed in. It places it in the
    path defined
    '''
    file_path = join(path, '%s.rst' % name)
    mod_file = open(file_path, 'w')

    mod_file.write('%s\n' % AUTOGEN)
    mod_file.write('%s\n' % name.title())
    mod_file.write('=' * len(name))
    mod_file.write('\n\n')
    mod_file.write('.. toctree::\n')
    mod_file.write('   :maxdepth: 1\n\n')
    mod_file.write('.. automodule:: %s.%s\n' % (package, name))
    mod_file.write('   :members:\n')
    mod_file.write('   :undoc-members:\n')
    mod_file.write('   :show-inheritance:\n')

def main():
    modules = dict(client=None, api=None)
    modules['client'] = get_module_names(MODULES_PATH)
    modules['api'] = get_module_names('%s/api' % MODULES_PATH)
    modules['client'].sort()
    modules['api'].sort()
    process_modules(modules)






if __name__ == '__main__':
    main()



#
# Copyright (c) 2015, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest
import itertools

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest


class TestApiNtp(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'ntp server 99.99.1.1'])
            else:
                dut.config(['ntp source Ethernet1', 'ntp server 99.99.1.1'])
            response = dut.api('ntp').get()
            self.assertIsNotNone(response)

    def test_create(self):
        intf = 'Ethernet1'
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['no ntp local-interface'])
            else:
                dut.config(['no ntp source'])
            response = dut.api('ntp').create(intf)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertEqual(response['source_interface'], intf)

    def test_delete(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1'])
            else:
                dut.config(['ntp source Ethernet1'])
            response = dut.api('ntp').delete()
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertIsNone(response['source_interface'])

    def test_default(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1'])
            else:
                dut.config(['ntp source Ethernet1'])
            response = dut.api('ntp').default()
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertIsNone(response['source_interface'])

    def test_set_source_interface(self):
        intf = 'Ethernet1'
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Loopback0'])
            else:
                dut.config(['ntp source Loopback0'])
            response = dut.api('ntp').set_source_interface(intf)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertEqual(response['source_interface'], intf)

    def test_add_server_single(self):
        server = '10.10.10.35'
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'no ntp'])
            else:
                dut.config(['ntp source Ethernet1', 'no ntp'])
            response = dut.api('ntp').add_server(server)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            keys = [x.keys() for x in response['servers']]
            keys = list(itertools.chain.from_iterable(keys))
            self.assertListEqual(keys, [server])

    def test_add_server_multiple(self):
        servers = ['10.10.10.37', '10.10.10.36', '10.10.10.34']
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'no ntp'])
            else:
                dut.config(['ntp source Ethernet1', 'no ntp'])
            for server in servers:
                response = dut.api('ntp').add_server(server)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            keys = [x.keys() for x in response['servers']]
            keys = list(itertools.chain.from_iterable(keys))
            self.assertListEqual(sorted(keys), sorted(servers))

    def test_add_server_prefer(self):
        server = '10.10.10.35'
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'no ntp'])
            else:
                dut.config(['ntp source Ethernet1', 'no ntp'])
            response = dut.api('ntp').add_server(server, prefer=False)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertIsNone(response['servers'][0][server])

            response = dut.api('ntp').add_server(server, prefer=True)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertEqual(response['servers'][0][server], 'prefer')

    def test_add_server_invalid(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'no ntp'])
            else:
                dut.config(['ntp source Ethernet1', 'no ntp'])
            with self.assertRaises(ValueError):
                dut.api('ntp').add_server(None)
                dut.api('ntp').add_server('')
                dut.api('ntp').add_server(' ')

    def test_remove_server(self):
        server = '10.10.10.35'
        servers = ['10.10.10.37', '10.10.10.36', '10.10.10.34']
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'no ntp',
                            'ntp server %s' % server])
            else:
                dut.config(['ntp source Ethernet1', 'no ntp',
                            'ntp server %s' % server])
            for addserver in servers:
                dut.config(['ntp server %s' % addserver])
            response = dut.api('ntp').remove_server(server)
            self.assertTrue(response)
            response = dut.api('ntp').get()
            keys = [x.keys() for x in response['servers']]
            keys = list(itertools.chain.from_iterable(keys))
            self.assertListEqual(sorted(keys), sorted(servers))

    def test_remove_all_servers(self):
        servers = ['10.10.10.37', '10.10.10.36', '10.10.10.34']
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config(['ntp local-interface Ethernet1', 'no ntp'])
            else:
                dut.config(['ntp source Ethernet1', 'no ntp'])
            for addserver in servers:
                dut.config(['ntp server %s' % addserver])
            response = dut.api('ntp').remove_all_servers()
            self.assertTrue(response)
            response = dut.api('ntp').get()
            self.assertEqual(response['servers'], [])


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import random_string
from systestlib import DutSystemTest


class TestApiSystem(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config('default hostname')
            resp = dut.api('system').get()
            keys = ['hostname', 'iprouting', 'banner_motd', 'banner_login']
            self.assertEqual(sorted(keys), sorted(resp.keys()))


    def test_get_with_period(self):
        for dut in self.duts:
            dut.config('hostname host.domain.net')
            response = dut.api('system').get()
            self.assertEqual(response['hostname'], 'host.domain.net')

    def test_get_check_hostname(self):
        for dut in self.duts:
            dut.config('hostname teststring')
            response = dut.api('system').get()
            self.assertEqual(response['hostname'], 'teststring')

    def test_get_check_banners(self):
        for dut in self.duts:
            motd_banner_value = random_string() + "\n"
            login_banner_value = random_string() + "\n"
            dut.config([dict(cmd="banner motd", input=motd_banner_value)])
            dut.config([dict(cmd="banner login", input=login_banner_value)])
            resp = dut.api('system').get()
            self.assertEqual(resp['banner_login'], login_banner_value.rstrip())
            self.assertEqual(resp['banner_motd'], motd_banner_value.rstrip())

    def test_get_banner_with_EOF(self):
        for dut in self.duts:
            motd_banner_value = '!!!newlinebaner\nSecondLIneEOF!!!newlinebanner\n'
            dut.config([dict(cmd="banner motd", input=motd_banner_value)])
            resp = dut.api('system').get()
            self.assertEqual(resp['banner_motd'], motd_banner_value.rstrip())

    def test_set_hostname_with_value(self):
        for dut in self.duts:
            dut.config('default hostname')
            value = random_string()
            response = dut.api('system').set_hostname(value)
            self.assertTrue(response, 'dut=%s' % dut)
            value = 'hostname %s' % value
            self.assertIn(value, dut.running_config)

    def test_set_hostname_with_no_value(self):
        for dut in self.duts:
            dut.config('hostname test')
            response = dut.api('system').set_hostname(disable=True)
            self.assertTrue(response, 'dut=%s' % dut)
            value = 'no hostname'
            self.assertIn(value, dut.running_config)

    def test_set_hostname_with_default(self):
        for dut in self.duts:
            dut.config('hostname test')
            response = dut.api('system').set_hostname(default=True)
            self.assertTrue(response, 'dut=%s' % dut)
            value = 'no hostname'
            self.assertIn(value, dut.running_config)

    def test_set_hostname_default_over_value(self):
        for dut in self.duts:
            dut.config('hostname test')
            response = dut.api('system').set_hostname(value='foo', default=True)
            self.assertTrue(response, 'dut=%s' % dut)
            value = 'no hostname'
            self.assertIn(value, dut.running_config)

    def test_set_iprouting_to_true(self):
        for dut in self.duts:
            dut.config('no ip routing')
            resp = dut.api('system').set_iprouting(True)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertNotIn('no ip rotuing', dut.running_config)

    def test_set_iprouting_to_false(self):
        for dut in self.duts:
            dut.config('ip routing')
            resp = dut.api('system').set_iprouting(False)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertIn('no ip routing', dut.running_config)

    def test_set_iprouting_to_no(self):
        for dut in self.duts:
            dut.config('ip routing')
            resp = dut.api('system').set_iprouting(disable=True)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertIn('no ip routing', dut.running_config)

    def test_set_iprouting_to_default(self):
        for dut in self.duts:
            dut.config('ip routing')
            resp = dut.api('system').set_iprouting(default=True)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertIn('no ip routing', dut.running_config)

    def test_set_hostname_with_period(self):
        for dut in self.duts:
            dut.config('hostname localhost')
            response = dut.api('system').set_hostname(value='host.domain.net')
            self.assertTrue(response, 'dut=%s' % dut)
            value = 'hostname host.domain.net'
            self.assertIn(value, dut.running_config)

    def test_set_banner_motd(self):
        for dut in self.duts:
            banner_value = random_string()
            dut.config([dict(cmd="banner motd",
                             input=banner_value)])
            self.assertIn(banner_value, dut.running_config)
            banner_api_value = random_string()
            resp = dut.api('system').set_banner("motd", banner_api_value)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertIn(banner_api_value, dut.running_config)

    def test_set_banner_motd_donkey(self):
        for dut in self.duts:
            donkey_chicken = r"""
                                  /\          /\
                                 ( \\        // )
                                  \ \\      // /
                                   \_\\||||//_/
                                    \/ _  _ \
                                   \/|(o)(O)|
                                  \/ |      |
              ___________________\/  \      /
             //                //     |____|       Cluck cluck cluck!
            //                ||     /      \
           //|                \|     \ 0  0 /
          // \       )         V    / \____/
         //   \     /        (     /
        ""     \   /_________|  |_/
               /  /\   /     |  ||
              /  / /  /      \  ||
              | |  | |        | ||
              | |  | |        | ||
              |_|  |_|        |_||
               \_\  \_\        \_\\
            """

            resp = dut.api('system').set_banner("motd", donkey_chicken)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertIn(donkey_chicken, dut.running_config)

    def test_set_banner_motd_default(self):
        for dut in self.duts:
            dut.config([dict(cmd="banner motd",
                             input="!!!!REMOVE BANNER TEST!!!!")])
            dut.api('system').set_banner('motd', None, True)
            self.assertIn('no banner motd', dut.running_config)

    def test_set_banner_login(self):
        for dut in self.duts:
            banner_value = random_string()
            dut.config([dict(cmd="banner login",
                             input=banner_value)])
            self.assertIn(banner_value, dut.running_config)
            banner_api_value = random_string()
            resp = dut.api('system').set_banner("login", banner_api_value)
            self.assertTrue(resp, 'dut=%s' % dut)
            self.assertIn(banner_api_value, dut.running_config)
            config_login_banner = dut.api('system').get()['banner_login']
            self.assertTrue(config_login_banner, banner_api_value.strip())

    def test_set_banner_login_default(self):
        for dut in self.duts:
            dut.config([dict(cmd="banner login",
                             input="!!!!REMOVE LOGIN BANNER TEST!!!!")])
            dut.api('system').set_banner('login', None, True)
            self.assertIn('no banner login', dut.running_config)

    def test_set_banner_login_negate(self):
        for dut in self.duts:
            dut.config([dict(cmd="banner login",
                             input="!!!!REMOVE LOGIN BANNER TEST!!!!")])
            dut.api('system').set_banner('login', None, False, True)
            self.assertIn('no banner login', dut.running_config)




if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from systestlib import DutSystemTest


class TestApiMlag(DutSystemTest):

    def test_get(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel1-2000',
                        'default mlag configuration'])
            response = dut.api('mlag').get()
            config = dict(domain_id=None, local_interface=None,
                          peer_link=None, peer_address=None, shutdown=False)
            values = dict(config=config, interfaces=dict())
            self.assertEqual(values, response)

    def test_set_domain_id_with_value(self):
        for dut in self.duts:
            dut.config('default mlag configuration')
            api = dut.api('mlag')
            self.assertIn('no domain-id', api.get_block('mlag configuration'))
            for domid in ['test_domain_id', 'test.dom-id', 'test domain id']:
                result = dut.api('mlag').set_domain_id(domid)
                self.assertTrue(result)
                self.assertIn('domain-id %s' % domid, api.get_block('mlag configuration'))

    def test_set_domain_id_with_no_value(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'domain-id test'])
            api = dut.api('mlag')
            self.assertIn('domain-id test', api.get_block('mlag configuration'))
            result = dut.api('mlag').set_domain_id(disable=True)
            self.assertTrue(result)
            self.assertIn('no domain-id', api.get_block('mlag configuration'))

    def test_set_domain_id_with_default(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'domain-id test'])
            api = dut.api('mlag')
            self.assertIn('domain-id test', api.get_block('mlag configuration'))
            result = dut.api('mlag').set_domain_id(default=True)
            self.assertTrue(result)
            self.assertIn('no domain-id', api.get_block('mlag configuration'))

    def test_set_local_interface_with_value(self):
        for dut in self.duts:
            dut.config('default mlag configuration')
            api = dut.api('mlag')
            self.assertIn('no local-interface', api.get_block('mlag configuration'))
            result = dut.api('mlag').set_local_interface('Vlan1234')
            self.assertTrue(result)
            self.assertIn('local-interface Vlan1234', api.get_block('mlag configuration'))

    def test_set_local_interface_with_no_value(self):
        for dut in self.duts:
            dut.config(['interface Vlan1234', 'mlag configuration', 'local-interface Vlan1234'])
            api = dut.api('mlag')
            self.assertIn('local-interface Vlan1234', api.get_block('mlag configuration'))
            result = api.set_local_interface(disable=True)
            self.assertTrue(result)
            self.assertIn('no local-interface', api.get_block('mlag configuration'))

    def test_set_local_interface_with_default(self):
        for dut in self.duts:
            dut.config(['interface Vlan1234', 'mlag configuration', 'local-interface Vlan1234'])
            api = dut.api('mlag')
            self.assertIn('local-interface Vlan1234', api.get_block('mlag configuration'))
            result = api.set_local_interface(default=True)
            self.assertTrue(result)
            self.assertIn('no local-interface', api.get_block('mlag configuration'))

    def test_set_peer_address_with_value(self):
        for dut in self.duts:
            dut.config('default mlag configuration')
            api = dut.api('mlag')
            self.assertIn('no peer-address', api.get_block('mlag configuration'))
            result = dut.api('mlag').set_peer_address('1.2.3.4')
            self.assertTrue(result)
            self.assertIn('peer-address 1.2.3.4', api.get_block('mlag configuration'))

    def test_set_peer_address_with_no_value(self):
        for dut in self.duts:
            dut.config(['interface Vlan1234', 'ip address 1.2.3.1/24',
                        'mlag configuration', 'peer-address 1.2.3.4'])
            api = dut.api('mlag')
            self.assertIn('peer-address 1.2.3.4',
                          api.get_block('mlag configuration'))
            result = api.set_peer_address(disable=True)
            self.assertTrue(result)
            self.assertIn('no peer-address',
                          api.get_block('mlag configuration'))

    def test_set_peer_address_with_default(self):
        for dut in self.duts:
            dut.config(['interface Vlan1234', 'ip address 1.2.3.1/24',
                        'mlag configuration', 'peer-address 1.2.3.4'])
            api = dut.api('mlag')
            self.assertIn('peer-address 1.2.3.4',
                          api.get_block('mlag configuration'))
            result = api.set_peer_address(default=True)
            self.assertTrue(result)
            self.assertIn('no peer-address',
                          api.get_block('mlag configuration'))

    def test_set_peer_link_with_value(self):
        for dut in self.duts:
            dut.config('default mlag configuration')
            api = dut.api('mlag')
            self.assertIn('no peer-link', api.get_block('mlag configuration'))
            result = dut.api('mlag').set_peer_link('Ethernet1')
            self.assertTrue(result)
            self.assertIn('peer-link Ethernet1',
                          api.get_block('mlag configuration'))

    def test_set_peer_link_with_value_portchannel(self):
        for dut in self.duts:
            dut.config(['default mlag configuration',
                        'interface Port-Channel5'])
            api = dut.api('mlag')
            self.assertIn('no peer-link', api.get_block('mlag configuration'))
            result = dut.api('mlag').set_peer_link('Port-Channel5')
            self.assertTrue(result)
            self.assertIn('peer-link Port-Channel5', api.get_block('mlag configuration'))

    def test_set_peer_link_with_no_value(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'peer-link Ethernet1'])
            api = dut.api('mlag')
            self.assertIn('peer-link Ethernet1', api.get_block('mlag configuration'))
            result = api.set_peer_link(disable=True)
            self.assertTrue(result)
            self.assertIn('no peer-link', api.get_block('mlag configuration'))

    def test_set_peer_link_with_default(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'peer-link Ethernet1'])
            api = dut.api('mlag')
            self.assertIn('peer-link Ethernet1',
                          api.get_block('mlag configuration'))
            result = api.set_peer_link(default=True)
            self.assertTrue(result)
            self.assertIn('no peer-link', api.get_block('mlag configuration'))

    def test_set_shutdown_with_true(self):
        for dut in self.duts:
            dut.config('default mlag configuration')
            api = dut.api('mlag')
            self.assertIn('no shutdown', api.get_block('mlag configuration'))
            result = api.set_shutdown(True)
            self.assertTrue(result)
            self.assertIn('shutdown', api.get_block('mlag configuration'))

    def test_set_shutdown_with_false(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'shutdown'])
            api = dut.api('mlag')
            self.assertIn('shutdown', api.get_block('mlag configuration'))
            result = api.set_shutdown(False)
            self.assertTrue(result)
            self.assertIn('no shutdown', api.get_block('mlag configuration'))

    def test_set_shutdown_with_no_value(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'shutdown'])
            api = dut.api('mlag')
            self.assertIn('shutdown', api.get_block('mlag configuration'))
            result = api.set_shutdown(disable=True)
            self.assertTrue(result)
            self.assertIn('no shutdown', api.get_block('mlag configuration'))

    def test_set_shutdown_with_default(self):
        for dut in self.duts:
            dut.config(['mlag configuration', 'shutdown'])
            api = dut.api('mlag')
            self.assertIn('shutdown', api.get_block('mlag configuration'))
            result = api.set_shutdown(default=True)
            self.assertTrue(result)
            self.assertIn('no shutdown', api.get_block('mlag configuration'))

    def test_set_mlag_id_with_value(self):
        for dut in self.duts:
            dut.config('no interface Port-Channel10')
            api = dut.api('mlag')
            self.assertIsNone(api.get_block('interface Port-Channel10'))
            result = api.set_mlag_id('Port-Channel10', '100')
            self.assertTrue(result)
            self.assertIn('mlag 100', api.get_block('interface Port-Channel10'))

    def test_set_mlag_id_with_no_value(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel10',
                        'interface Port-Channel10', 'mlag 100'])
            api = dut.api('mlag')
            self.assertIn('mlag 100',
                          api.get_block('interface Port-Channel10'))
            result = api.set_mlag_id('Port-Channel10', disable=True)
            self.assertTrue(result)
            self.assertIn('no mlag', api.get_block('interface Port-Channel10'))

    def test_set_mlag_id_with_default(self):
        for dut in self.duts:
            dut.config(['no interface Port-Channel10',
                        'interface Port-Channel10', 'mlag 100'])
            api = dut.api('mlag')
            self.assertIn('mlag 100',
                          api.get_block('interface Port-Channel10'))
            result = api.set_mlag_id('Port-Channel10', default=True)
            self.assertTrue(result)
            self.assertIn('no mlag', api.get_block('interface Port-Channel10'))


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Arista Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#   Neither the name of Arista Networks nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ARISTA NETWORKS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import os
import unittest

import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../lib'))

from testlib import random_int, random_string, get_fixture
from mock import patch

import pyeapi.client
import pyeapi.eapilib


class TestClient(unittest.TestCase):

    def setUp(self):
        pyeapi.client.load_config(filename=get_fixture('dut.conf'))
        config = pyeapi.client.config

        self.duts = list()
        for name in config.sections():
            if name.startswith('connection:') and 'localhost' not in name:
                name = name.split(':')[1]
                dut = pyeapi.client.connect_to(name)
                self.duts.append(dut)
                if dut._enablepwd is not None:
                    # If enable password defined for dut, set the
                    # enable password on the dut and clear it on tearDown
                    if dut.version_number >= '4.23':
                        dut.config("enable password %s" % dut._enablepwd)
                    else:
                        dut.config("enable secret %s" % dut._enablepwd)

    def test_unauthorized_user(self):
        error_string = ('Unauthorized. Unable to authenticate user: Bad'
                        ' username/password combination')
        for dut in self.duts:
            temp_node = pyeapi.connect(host=dut.settings['host'],
                                       transport=dut.settings['transport'],
                                       username='wrong', password='nope',
                                       port=dut.settings['port'],
                                       return_node=True)
            try:
                temp_node.run_commands('show version')
            except pyeapi.eapilib.ConnectionError as err:
                self.assertEqual(err.message, error_string)

    def test_populate_version_properties(self):
        for dut in self.duts:
            result = dut.run_commands('show version')
            self.assertEqual(dut.version, result[0]['version'])
            self.assertIn(dut.model, result[0]['modelName'])
            self.assertIn(dut.version_number, result[0]['version'])

    def test_enable_single_command(self):
        for dut in self.duts:
            result = dut.run_commands('show version')
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), 1, 'dut=%s' % dut)

    def test_enable_single_extended_command(self):
        for dut in self.duts:
            result = dut.run_commands({'cmd': 'show cvx', 'revision': 1})
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), 1, 'dut=%s' % dut)
            self.assertTrue('clusterMode' not in result[0].keys())

            result2 = dut.run_commands({'cmd': 'show cvx', 'revision': 2})
            self.assertIsInstance(result2, list, 'dut=%s' % dut)
            self.assertEqual(len(result2), 1, 'dut=%s' % dut)
            self.assertTrue('clusterMode' in result2[0].keys())

    def test_enable_single_unicode_command(self):
        for dut in self.duts:
            result = dut.run_commands(u'show version')
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), 1, 'dut=%s' % dut)

    def test_no_enable_single_command(self):
        for dut in self.duts:
            result = dut.run_commands('show version', 'json', send_enable=False)
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), 1, 'dut=%s' % dut)

    def test_no_enable_single_command_no_auth(self):
        for dut in self.duts:
            with self.assertRaises(pyeapi.eapilib.CommandError):
                dut.run_commands(['disable', 'show running-config'], 'json', send_enable=False)

    def test_enable_multiple_commands(self):
        for dut in self.duts:
            commands = list()
            for i in range(1, random_int(10, 200)):
                commands.append('show version')
            result = dut.run_commands(commands[:])
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), len(commands), 'dut=%s' % dut)

    def test_enable_multiple_unicode_commands(self):
        for dut in self.duts:
            commands = list()
            for i in range(1, random_int(10, 200)):
                commands.append(u'show version')
            result = dut.enable(commands[:])
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), len(commands), 'dut=%s' % dut)

    def test_config_single_command(self):
        for dut in self.duts:
            hostname = 'hostname %s' % random_string(5, 50)
            result = dut.config(hostname)
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), 1, 'dut=%s' % dut)
            self.assertEqual(result[0], {}, 'dut=%s' % dut)

            result = dut.run_commands('show running-config | include %s$' %
                                      hostname, 'text')
            self.assertEqual(result[0]['output'].strip(), hostname)

    def test_config_single_multiline_command(self):
        for dut in self.duts:
            # Clear any current banner
            dut.config('no banner login')

            banner = 'This is a new banner\nwith different lines!!!'
            cmd = 'banner login MULTILINE:%s' % banner
            result = dut.config(cmd)
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), 1, 'dut=%s' % dut)
            self.assertEqual(result[0], {}, 'dut=%s' % dut)
            result = dut.run_commands('show banner login', 'text')
            self.assertEqual(result[0]['output'].strip().split('\n'),
                             banner.split('\n'))

    def test_config_multiple_commands(self):
        for dut in self.duts:
            commands = list()
            for i in range(1, random_int(10, 200)):
                commands.append('hostname %s' % random_string(5, 20))
            result = dut.config(commands[:])
            self.assertIsInstance(result, list, 'dut=%s' % dut)
            self.assertEqual(len(result), len(commands), 'dut=%s' % dut)

    def test_multiple_requests(self):
        for dut in self.duts:
            for i in range(1, random_int(10, 200)):
                result = dut.run_commands('show version')
                self.assertIsInstance(result, list, 'dut=%s' % dut)
                self.assertEqual(len(result), 1, 'dut=%s' % dut)

    def test_get_block(self):
        # Verify get_block using a config string returns correct value
        for dut in self.duts:
            api = dut.api('interfaces')
            config = api.config
            running = api.get_block('interface Ethernet1')
            txtstr = api.get_block('interface Ethernet1', config=config)
            self.assertEqual(running, txtstr)

    def test_get_block_none(self):
        # Verify get_block using a config string where match fails returns None
        for dut in self.duts:
            api = dut.api('interfaces')
            txtstr = api.get_block('interface Ethernet1', config='config')
            self.assertEqual(txtstr, None)

    def test_execute_with_autocomplete(self):
        # There are some versions of EOS before 4.17.x that have the
        # autocomplete feature available. If system tests are run on one of
        # those version of EOS this system test will fail.
        for dut in self.duts:
            version = self._dut_eos_version(dut)
            version = version.split('.')
            if int(version[0]) >= 4 and int(version[1]) >= 17:
                result = dut.connection.execute(['sh ver'], encoding='json',
                                                autoComplete=True)
                self.assertIn('version', result['result'][0])
            else:
                # Verify exception thrown for EOS version that does not
                # support autoComplete parameter with EAPI
                with self.assertRaises(pyeapi.eapilib.CommandError):
                    dut.connection.execute(['sh ver'], encoding='json',
                                           autoComplete=True)

    def test_execute_with_expandaliases(self):
        # There are some versions of EOS before 4.17.x that have the
        # expandaliases feature available. If system tests are run on one of
        # those version of EOS this system test will fail.
        for dut in self.duts:
            # configure an alias for show version command
            dut.config(['alias test show version'])
            version = self._dut_eos_version(dut)
            version = version.split('.')
            if int(version[0]) >= 4 and int(version[1]) >= 17:
                result = dut.connection.execute(['test'], encoding='json',
                                                expandAliases=True)
                self.assertIn('version', result['result'][0])
            else:
                # Verify exception thrown for EOS version that does not
                # support expandAliases parameter with EAPI
                with self.assertRaises(pyeapi.eapilib.CommandError):
                    dut.connection.execute(['test'], encoding='json',
                                           expandAliases=True)

    @patch('pyeapi.eapilib._LOGGER.exception')
    def test_execute_socket_timeout_error(self, logexception):
        for dut in self.duts:
            self.assertEqual(dut.connection.transport.timeout, 60)
            dut.connection.transport.timeout = 0.001
            try:
                dut.connection.execute(['show version'], encoding='json')
            except pyeapi.eapilib.ConnectionError as err:
                error_msg = 'Socket error during eAPI connection: timed out'
                self.assertEqual(err.message, error_msg)
            logexception.assert_called_once()
            dut.connection.transport.timeout = 60

    def _dut_eos_version(self, dut):
        result = dut.connection.execute(['show version'], encoding='json')
        return result['result'][0]['version']

    def tearDown(self):
        for dut in self.duts:
            if dut.version_number >= '4.23':
                dut.config("no enable password")
            else:
                dut.config("no enable secret")


class TestNode(unittest.TestCase):

    def setUp(self):
        pyeapi.client.load_config(filename=get_fixture('dut.conf'))
        config = pyeapi.client.config

        self.duts = list()
        for name in config.sections():
            if name.startswith('connection:') and 'localhost' not in name:
                name = name.split(':')[1]
                self.duts.append(pyeapi.client.connect_to(name))

    def test_exception_trace(self):
        # Send commands that will return an error and validate the errors

        # General format of an error message:
        rfmt = r'Error \[%d\]: CLI command \d+ of \d+ \'.*\' failed: %s \[%s\]'
        # Design error tests
        cases = []
        # Send an incomplete command
        cases.append(('show run', rfmt
                      % (1002, 'invalid command',
                         r'incomplete token \(at token \d+: \'.*\'\)')))
        # Send a mangled command
        cases.append(('shwo version', rfmt
                      % (1002, 'invalid command',
                         r'Invalid input \(at token \d+: \'.*\'\)')))
        # Send a command that cannot be run through the api
        # note the command for reload looks to change in new EOS
        # in 4.15 the  reload now is replaced with 'force' if you are
        # testing some DUT running older code and this test fails
        # change the error message to the following:
        # To reload the machine over the API, please use 'reload now' instead
        cases.append(('reload', rfmt
                      % (1004, 'incompatible command',
                         'Command not permitted via API access..*')))
        # Send a command that has insufficient priv
        cases.append(('show running-config', rfmt
                      % (1002, 'invalid command',
                         r'Invalid input \(privileged mode required\)')))

        for dut in self.duts:
            for (cmd, regex) in cases:
                try:
                    # Insert the error in list of valid commands
                    if cmd != "show running-config":
                        dut.enable(['show version', cmd, 'show hostname'],
                                   strict=True)
                    else:
                        dut.enable(['disable', 'show version', cmd],
                                   strict=True, send_enable=False)

                    self.fail('A CommandError should have been raised')
                except pyeapi.eapilib.CommandError as exc:
                    # Validate the properties of the exception
                    if cmd != 'show running-config':
                        self.assertEqual(len(exc.trace), 4)
                    else:
                        self.assertEqual(len(exc.trace), 3)
                    self.assertIsNotNone(exc.command_error)
                    self.assertIsNotNone(exc.output)
                    self.assertIsNotNone(exc.commands)
                    self.assertRegexpMatches(exc.message, regex)


if __name__ == '__main__':
    unittest.main()
